
# q-expansion of newform 392.3.k.f, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 392
weight = 3
poly_data = [4, 0, 2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 1, 0], 2], [[0, 0, 0, 1], 2]]

hecke_ring_character_values = [[295, [-1, 0, 0, 0]], [197, [-1, 0, 0, 0]], [297, [-1, 0, -1, 0]]]
aps_data = [[-2, 0, -2, 0], [0, -1, -4, -1], [0, 0, 0, 0], [0, 0, 0, 0], [0, -12, 0, -12], [0, 0, 0, 0], [0, -1, 24, -1], [-12, -17, -12, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [48, 0, 0, 23], [0, 0, 0, -60], [0, 0, 0, 0], [0, 0, 0, 0], [0, 41, -60, 41], [0, 0, 0, 0], [0, 0, -62, 0], [0, 0, 0, 0], [0, -71, 24, -71], [0, 0, 0, 0], [-36, 0, 0, 79], [72, -73, 72, 0], [-120, 0, 0, -47], [0, 0, 0, 0], [0, 0, 0, 0], [178, 0, 178, 0], [0, 0, 0, 0], [0, 0, 0, -144], [0, 0, 0, 0], [-180, -31, -180, 0], [0, -96, 0, -96], [132, 0, 0, -103], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -36, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, -34, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -264, 0, -264], [0, 0, 0, 0], [0, 0, 0, 0], [226, 0, 0, 0], [0, 0, 0, 0], [0, 223, -60, 223], [0, 0, 0, 0], [0, -120, 0, 0], [0, 0, 0, 0], [0, 97, -312, 97], [132, 0, 0, 233], [240, 193, 240, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 360], [0, 41, -396, 41], [0, 0, 0, 0], [-204, 0, 0, -271], [0, 0, 0, 0], [240, 263, 240, 0], [0, 0, 0, 0], [0, 468, 0, 0], [478, 0, 0, 0], [0, 156, 0, 156], [0, 0, 0, 0], [0, 97, -480, 97], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -228], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [766, 0, 766, 0], [0, 167, 528, 167], [468, 0, 0, -257], [0, 0, 0, 0], [0, 0, 0, 0], [-456, 0, 0, 289], [0, 0, 0, 0], [878, 0, 878, 0], [-866, 0, 0, 0], [0, -624, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [660, -17, 660, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-782, 0, 0, 0], [-802, 0, -802, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 503, 192, 503], [-684, -199, -684, 0], [0, 0, 0, 0], [0, 0, 0, 276], [0, 0, 0, 0], [0, -113, 780, -113], [626, 0, 626, 0], [0, 660, 0, 660], [0, -1, -816, -1], [-204, 0, 0, 569], [576, 431, 576, 0], [0, 0, 0, 0], [552, 0, 0, 457], [0, 0, 0, 0], [0, 0, 0, 0], [-334, 0, 0, 0], [0, -281, 780, -281], [0, 0, 0, 0], [0, 0, -482, 0], [300, 0, 0, -607], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 612], [0, 0, 0, 0], [0, 0, 0, 360], [0, 0, 0, 0], [0, 0, -398, 0], [828, -367, 828, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -1020, 0, -1020], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-432, 697, -432, 0], [-792, 0, 0, 527], [0, 0, 0, 0], [0, -463, -900, -463], [0, 0, 0, 0], [0, 240, 0, 240], [1140, 0, 0, -89], [0, 0, 0, 0], [0, 0, 0, 0], [1262, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 601, 864, 601], [-348, 823, -348, 0], [0, 0, 0, 0], [0, 0, 0, 0], [720, 0, 0, -719], [-1762, 0, 0, 0], [0, 0, 0, 0], [0, 0, -1714, 0], [0, 0, 0, 0], [0, 0, 0, 0], [1080, 529, 1080, 0], [1224, 0, 0, 359], [0, 0, 0, 0], [0, 1140, 0, 0], [142, 0, 0, 0], [0, 0, 0, 0], [-1188, 487, -1188, 0], [0, -264, 0, -264], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0]]
